import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 * This class will read and parse the lines (records) from the specified file.
 */
public class ProcessInputFile
{
   private JFrame       parent;
   private ArrayList    al = new ArrayList();

   /**
    * The constructor
    * @param parent
    */
   public ProcessInputFile(JFrame parent)
   {
      super();
      this.parent = parent;
   }

   /**
    * Read the file with the IP addresses. One IP address per line.
    * @param fileName
    * @return ArrayList
    * @throws FileNotFoundException, IOException
    */
   public void readFile(String fileName) throws FileNotFoundException
   {
      String line;
      BufferedReader in = null;
      int lineNumber = 1;

      in = new BufferedReader(new FileReader(fileName));

      try
      {
         while ((line = in.readLine()) != null)
         {
            if (line.trim().startsWith("#"))
            { /* comment */
            }
            else if (line.trim().startsWith(";"))
            { /* comment */
            }
            else if ("".equals(line.trim()))
            { /* blank */
            }
            /* does it have a range [...] */
            else if ( (line.indexOf('[') != -1) || (line.indexOf(']') != -1) )
            {
               int x = line.lastIndexOf('.');
               int sb = line.indexOf('[');
               int eb = line.indexOf(']');
               int p = line.indexOf('-');
               int sNum = 0;
               int eNum = 0;

               /* dash must be after last dot */
               if ( (x > p) || (sb == -1) || (eb == -1) || (x == -1) || (p == -1) )
               {
                  JOptionPane.showMessageDialog(parent,
                                                "Line # " + lineNumber + " is invalid: "+line,
                                                "File Error",
                                                JOptionPane.ERROR_MESSAGE);

                  continue;
               }

               String prefix = line.substring(0,x+1);

               try
               {
                  sNum = Integer.parseInt(line.substring(sb+1,p));
               }
               catch (NumberFormatException e)
               {
                  JOptionPane.showMessageDialog(parent,
                                                "Invalid number: "+line.substring(sb+1,p)+" on line # "+lineNumber,
                                                "File Error",
                                                JOptionPane.ERROR_MESSAGE);

                  continue; // not a number
               }

               try
               {
                  eNum = Integer.parseInt(line.substring(p+1,eb));
               }
               catch (NumberFormatException e)
               {
                  JOptionPane.showMessageDialog(parent,
                                                "Invalid number: "+line.substring(p+1,eb)+" on line # "+lineNumber,
                                                "File Error",
                                                JOptionPane.ERROR_MESSAGE);

                  continue; // not a number
               }

               for (int i=sNum; i <= eNum; i++)
               {
                  al.add(createNewRowOfObjects(new String(prefix + i)));
               }
            }
            else /* just a single line */
            {
               al.add(createNewRowOfObjects(line));
            }

            lineNumber++;
         }
      }
      catch (IOException e)
      {
         JOptionPane.showMessageDialog(parent,
                                       "IOException: "+e.getLocalizedMessage(),
                                       "File Error",
                                       JOptionPane.ERROR_MESSAGE);
      }
      finally
      {
         if (in != null)
         {
            try
            {
               in.close();
            }
            catch (IOException e)
            {
               JOptionPane.showMessageDialog(parent,
                                             "Close IOException: "+e.getLocalizedMessage(),
                                             "File Error",
                                             JOptionPane.ERROR_MESSAGE);
            }
         }
      }

   }

   /**
    * Create a row of objects for the table.
    * @param line
    */
   private Object[] createNewRowOfObjects(String line)
   {
      Object[] myObjects = new Object[3];
      myObjects[0] = ServerStatus.OFFLINE;

      int x = line.indexOf(' ');

      if (x == -1)
      {
         myObjects[1] = new String(line);
         myObjects[2] = new String("");
      }
      else
      {
         myObjects[1] = new String(line.substring(0, x).trim());
         myObjects[2] = new String(line.substring(x+1).trim());
      }

      return myObjects;
   }

   /**
    * Retrieve the ArrayList that contains the row objects.
    * @return ArrayList
    */
   public ArrayList getRowObjects()
   {
      return al;
   }
}
